"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoConnector = void 0;
const mongodb_1 = require("mongodb");
class MongoConnector {
    constructor(url, dbName) {
        this.url = url;
        this.dbName = dbName;
    }
    connect(started) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.client = yield mongodb_1.MongoClient.connect(this.url);
                this.db = yield this.client.db(this.dbName);
                if (started) {
                    started();
                }
            }
            catch (ex) {
                console.log(`Connection Error to ${this.url} / ${this.dbName}: ${ex.message}`);
                throw new Error(`Connection Error to ${this.url}/${this.dbName}`);
            }
        });
    }
    disconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.close();
        });
    }
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.db.dropDatabase();
            yield this.client.close();
        });
    }
    collection(name) {
        return this.db.collection(name);
    }
}
exports.MongoConnector = MongoConnector;
exports.default = MongoConnector;
