"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopServer = exports.stopDatabase = exports.getConnector = exports.itWithServer = exports.withServer = exports.itWithContext = exports.withContext = exports.itWithEntity = exports.withEntity = exports.getDb = exports.config = void 0;
const mongodb_1 = require("mongodb");
const mongo_entity_1 = require("./mongo_entity");
const mongodb_memory_server_1 = require("mongodb-memory-server");
let customGlobal = global;
customGlobal.__mongoConnectorConfig = {
    db: null,
    client: null,
    name: "",
    host: "127.0.0.1",
    port: "27017",
    initContext: null,
    testServer: null,
    verbose: false,
    dbName: () => "tmp" + Math.floor(Math.random() * 10000),
};
let glob = customGlobal.__mongoConnectorConfig;
function config({ mongoHost, testDbName, mongoPort, verbose, initContext: ic, testServer: ts, }) {
    glob.host = mongoHost || glob.host;
    glob.port = mongoPort || glob.port;
    glob.initContext = ic || glob.initContext;
    glob.testServer = ts || glob.testServer;
    glob.verbose = verbose;
    glob.dbName = testDbName || glob.dbName;
}
exports.config = config;
function getDb() {
    return __awaiter(this, void 0, void 0, function* () {
        if (glob.db) {
            return glob.db;
        }
        const mongod = yield mongodb_memory_server_1.MongoMemoryServer.create();
        const uri = mongod.getUri();
        glob.name = glob.dbName();
        console.log("New connection to " + glob.name + " " + uri);
        let client = yield mongodb_1.MongoClient.connect(uri);
        glob.db = yield client.db(glob.name);
        glob.client = client;
        global.db = glob.db;
        return glob.db;
    });
}
exports.getDb = getDb;
function withEntity(test, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const connector = yield getConnector();
        let entities = [];
        if (options && options.entities) {
            for (let i = 0; i < options.entities.length; i++) {
                const option = options.entities[i];
                const name = option.name;
                const type = option.type ? option.type : mongo_entity_1.default;
                let entity = new type(connector, name ? name : `test_${i}`);
                entities.push(entity);
                if (options.entities[i].data) {
                    for (let data of options.entities[i].data) {
                        yield entity.insertOne(data);
                    }
                }
            }
        }
        else {
            entities.push(new mongo_entity_1.default(connector, "test"));
        }
        try {
            yield test(...entities);
        }
        catch (ex) {
            throw ex;
        }
        finally {
            yield entities.forEach((e) => e.deleteMany({}));
        }
    });
}
exports.withEntity = withEntity;
function itWithEntity(name, func, options) {
    it(name, function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield withEntity((context) => __awaiter(this, void 0, void 0, function* () {
                yield func(context);
            }), options);
        });
    });
}
exports.itWithEntity = itWithEntity;
const fakeConnector = {
    collection() {
        return {
            deleteMany() { },
            insert() { },
            find() { },
            findOne() { },
        };
    },
};
function withContext(test, initContextFn, disconnected = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const connector = disconnected ? fakeConnector : yield getConnector();
        glob.initContext = initContextFn || glob.initContext;
        if (!glob.initContext) {
            throw new Error("No initContext provided, please pass as a parameter or use global config");
        }
        const context = glob.initContext(connector);
        try {
            yield test(context);
        }
        catch (ex) {
            throw ex;
        }
        finally {
            for (let key of Object.keys(context)) {
                if (context[key] && context[key].dispose) {
                    yield context[key].dispose({}, true);
                }
            }
        }
    });
}
exports.withContext = withContext;
function itWithContext(name, func, initContextFn, disconnected) {
    it(name, function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield withContext((context) => __awaiter(this, void 0, void 0, function* () {
                yield func(context);
            }), initContextFn, disconnected);
        });
    });
}
exports.itWithContext = itWithContext;
function withServer(test, server) {
    return __awaiter(this, void 0, void 0, function* () {
        if (server) {
            glob.testServer = server;
        }
        else {
            server = glob.testServer;
        }
        if (!server) {
            throw new Error("No server provided, please pass as a parameter or use global config");
        }
        if (!server.started) {
            yield server.startTest();
        }
        try {
            yield test(server);
        }
        catch (ex) {
            throw ex;
        }
        finally {
            for (let key of Object.keys(server.context)) {
                if (server.context[key] && server.context[key].dispose) {
                    yield server.context[key].dispose({}, true);
                }
            }
        }
    });
}
exports.withServer = withServer;
function itWithServer(name, func, server) {
    it(name, function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield withServer((context) => __awaiter(this, void 0, void 0, function* () {
                yield func(context);
            }), server);
        });
    });
}
exports.itWithServer = itWithServer;
function getConnector() {
    return __awaiter(this, void 0, void 0, function* () {
        let myDb = yield getDb();
        return {
            collection(name) {
                return myDb.collection(name);
            },
        };
    });
}
exports.getConnector = getConnector;
function stopDatabase() {
    return __awaiter(this, void 0, void 0, function* () {
        let myDb = glob.db;
        let client = glob.client;
        glob.db = null;
        if (myDb) {
            yield myDb.dropDatabase();
            yield client.close();
        }
    });
}
exports.stopDatabase = stopDatabase;
function stopServer() {
    if (!glob.testServer) {
        console.error("No server to stop!");
    }
    console.log("Stopping test server");
    glob.testServer.stopTest();
}
exports.stopServer = stopServer;
